@@ -26,12 +26,15 @@
 #include "katepartdebug.h"
 
 #include <QBrush>
+#include <QPaintEngine>
 #include <QPainter>
 #include <QPainterPath>
 #include <QRegularExpression>
 #include <QStack>
 #include <QtMath> // qCeil
 
+#include <optional>
+
 static const QChar tabChar(QLatin1Char('\t'));
 static const QChar spaceChar(QLatin1Char(' '));
 static const QChar nbSpaceChar(0xa0); // non-breaking space
@@ -264,8 +267,11 @@ void KateRenderer::paintTabstop(QPainter &paint, qreal x, qreal y) const
     paint.setPen(penBackup);
 }
 
-void KateRenderer::paintSpace(QPainter &paint, qreal x, qreal y) const
+void KateRenderer::paintSpaces(QPainter &paint, const QPointF *points, const int count) const
 {
+    if (count == 0) {
+        return;
+    }
     QPen penBackup(paint.pen());
     QPen pen(config()->tabMarkerColor());
 
@@ -273,7 +279,7 @@ void KateRenderer::paintSpace(QPainter &paint, qreal x, qreal y) const
     pen.setCapStyle(Qt::RoundCap);
     paint.setPen(pen);
     paint.setRenderHint(QPainter::Antialiasing, true);
-    paint.drawPoint(QPointF(x, y));
+    paint.drawPoints(points, count);
     paint.setPen(penBackup);
     paint.setRenderHint(QPainter::Antialiasing, false);
 }
@@ -628,12 +634,25 @@ void KateRenderer::assignSelectionBrushesFromAttribute(QTextLayout::FormatRange
     }
 }
 
-void KateRenderer::paintTextLineSelection(QPainter &paint, KateLineLayoutPtr layout, const QVector<QTextLayout::FormatRange> &selRanges)
+void KateRenderer::paintTextBackground(QPainter &paint, KateLineLayoutPtr layout, const QVector<QTextLayout::FormatRange> &selRanges, const QBrush &brush) const
 {
-    QPainterPath p;
+    const bool rtl = layout->isRightToLeft();
+
     for (const auto &sel : selRanges) {
         const int s = sel.start;
         const int e = sel.start + sel.length;
+        QBrush br;
+
+        // Prefer using the brush supplied by user
+        if (brush != Qt::NoBrush) {
+            br = brush;
+        } else if (sel.format.background() != Qt::NoBrush) {
+            // Otherwise use the brush in format
+            br = sel.format.background();
+        } else {
+            // skip if no brush to fill with
+            continue;
+        }
 
         const int startViewLine = layout->viewLineForColumn(s);
         const int endViewLine = layout->viewLineForColumn(e);
@@ -643,25 +662,38 @@ void KateRenderer::paintTextLineSelection(QPainter &paint, KateLineLayoutPtr lay
             const int endX = cursorToX(l, e);
             const int y = startViewLine * lineHeight();
             QRect r(startX, y, (endX - startX), lineHeight());
-            p.addRect(r);
+            paint.fillRect(r, br);
         } else {
+            QPainterPath p;
             for (int l = startViewLine; l <= endViewLine; ++l) {
                 auto kateLayout = layout->viewLine(l);
                 int sx = 0;
-                int width = kateLayout.lineLayout().naturalTextWidth();
+                int width = rtl ? kateLayout.lineLayout().width() : kateLayout.lineLayout().naturalTextWidth();
+
                 if (l == startViewLine) {
-                    sx = kateLayout.lineLayout().cursorToX(s);
+                    if (rtl) {
+                        // For rtl, Rect starts at 0 and ends at selection start
+                        sx = 0;
+                        width = kateLayout.lineLayout().cursorToX(s);
+                    } else {
+                        sx = kateLayout.lineLayout().cursorToX(s);
+                    }
                 } else if (l == endViewLine) {
-                    width = kateLayout.lineLayout().cursorToX(e);
+                    if (rtl) {
+                        // Drawing will start at selection end, and end at the view border
+                        sx = kateLayout.lineLayout().cursorToX(e);
+                    } else {
+                        width = kateLayout.lineLayout().cursorToX(e);
+                    }
                 }
 
                 const int y = l * lineHeight();
                 QRect r(sx, y, width - sx, lineHeight());
                 p.addRect(r);
             }
+            paint.fillPath(p, br);
         }
     }
-    paint.fillPath(p, config()->selectionColor());
 }
 
 void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int xStart, int xEnd, const KTextEditor::Cursor *cursor, PaintTextLineFlags flags)
@@ -724,10 +756,28 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
             // set the pen color
             paint.setPen(attribute(KTextEditor::dsNormal)->foreground().color());
             // Draw the text :)
+
+            if (range->layout()->textOption().textDirection() == Qt::RightToLeft) {
+                // If the text is RTL, we draw text background ourselves
+                auto decos = decorationsForLine(range->textLine(), range->line(), false);
+                auto sr = view()->selectionRange();
+                auto c = config()->selectionColor();
+                int line = range->line();
+                // Remove "selection" format from decorations
+                // "selection" will get painted below
+                decos.erase(std::remove_if(decos.begin(),
+                                           decos.end(),
+                                           [sr, c, line](const QTextLayout::FormatRange &fr) {
+                                               return sr.overlapsLine(line) && sr.overlapsColumn(fr.start) && fr.format.background().color() == c;
+                                           }),
+                            decos.end());
+                paintTextBackground(paint, range, decos, Qt::NoBrush);
+            }
+
             if (drawSelection) {
                 additionalFormats = decorationsForLine(range->textLine(), range->line(), true);
                 if (hasCustomLineHeight()) {
-                    paintTextLineSelection(paint, range, additionalFormats);
+                    paintTextBackground(paint, range, additionalFormats, config()->selectionColor());
                 }
                 range->layout()->draw(&paint, QPoint(-xStart, 0), additionalFormats);
 
@@ -762,7 +812,7 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
 
             // draw an open box to mark non-breaking spaces
             const QString &text = range->textLine()->text();
-            int y = lineHeight() * i + fm.ascent() - fm.strikeOutPos();
+            int y = lineHeight() * i + m_fontAscent - fm.strikeOutPos();
             int nbSpaceIndex = text.indexOf(nbSpaceChar, line.lineLayout().xToCursor(xStart));
 
             while (nbSpaceIndex != -1 && nbSpaceIndex < line.endCol()) {
@@ -793,6 +843,11 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
                 const int trailingPos = showSpaces() == KateDocumentConfig::All ? 0 : qMax(range->textLine()->lastChar(), 0);
 
                 if (spaceIndex >= trailingPos) {
+                    QVarLengthArray<int, 32> spacePositions;
+                    // Adjust to visible contents
+                    spaceIndex = std::min(line.lineLayout().xToCursor(xEnd), spaceIndex);
+                    int visibleStart = line.lineLayout().xToCursor(xStart);
+
                     for (; spaceIndex >= line.startCol(); --spaceIndex) {
                         if (!text.at(spaceIndex).isSpace()) {
                             if (showSpaces() == KateDocumentConfig::Trailing) {
@@ -801,20 +856,42 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
                                 continue;
                             }
                         }
-
                         if (text.at(spaceIndex) != QLatin1Char('\t') || !showTabs()) {
-                            if (range->layout()->textOption().alignment() == Qt::AlignRight) { // Draw on left for RTL lines
-                                paintSpace(paint, line.lineLayout().cursorToX(spaceIndex) - xStart - spaceWidth() / 2.0, y);
-                            } else {
-                                paintSpace(paint, line.lineLayout().cursorToX(spaceIndex) - xStart + spaceWidth() / 2.0, y);
-                            }
+                            spacePositions << spaceIndex;
                         }
+
+                        if (spaceIndex < visibleStart) {
+                            break;
+                        }
+                    }
+
+                    QPointF prev;
+                    QVarLengthArray<QPointF, 32> spacePoints;
+                    const auto spaceWidth = this->spaceWidth();
+                    // reverse because we want to look at the spaces at the beginning of line first
+                    for (auto rit = spacePositions.rbegin(); rit != spacePositions.rend(); ++rit) {
+                        const int spaceIdx = *rit;
+                        qreal x;
+                        if (range->layout()->textOption().alignment() == Qt::AlignRight) {
+                            x = line.lineLayout().cursorToX(spaceIdx) - xStart - spaceWidth / 2.0;
+                        } else {
+                            x = (line.lineLayout().cursorToX(spaceIdx) - xStart) + (spaceWidth / 2.0);
+                        }
+                        const QPointF currentPoint(x, y);
+                        if (!prev.isNull() && currentPoint == prev) {
+                            break;
+                        }
+                        spacePoints << currentPoint;
+                        prev = QPointF(x, y);
+                    }
+                    if (!spacePoints.isEmpty()) {
+                        paintSpaces(paint, spacePoints.constData(), spacePoints.size());
                     }
                 }
             }
 
             if (showNonPrintableSpaces()) {
-                const int y = lineHeight() * i + fm.ascent();
+                const int y = lineHeight() * i + m_fontAscent;
 
                 static const QRegularExpression nonPrintableSpacesRegExp(
                     QStringLiteral("[\\x{2000}-\\x{200F}\\x{2028}-\\x{202F}\\x{205F}-\\x{2064}\\x{206A}-\\x{206F}]"));
@@ -909,6 +986,28 @@ void KateRenderer::paintTextLine(QPainter &paint, KateLineLayoutPtr range, int x
     }
 }
 
+static void drawCursor(const QTextLayout &layout, QPainter *p, const QPointF &pos, int cursorPosition, int width, const int height)
+{
+    if (!layout.isValidCursorPosition(cursorPosition)) {
+        cursorPosition = 0;
+    }
+    const QTextLine l = layout.lineForTextPosition(cursorPosition);
+    Q_ASSERT(l.isValid());
+    if (!l.isValid()) {
+        return;
+    }
+    const QPainter::CompositionMode origCompositionMode = p->compositionMode();
+    if (p->paintEngine()->hasFeature(QPaintEngine::RasterOpModes)) {
+        p->setCompositionMode(QPainter::RasterOp_NotDestination);
+    }
+
+    const QPointF position = pos + layout.position();
+    const qreal x = position.x() + l.cursorToX(cursorPosition);
+    const qreal y = l.lineNumber() * height;
+    p->fillRect(QRectF(x, y, (qreal)width, (qreal)height), p->pen().brush());
+    p->setCompositionMode(origCompositionMode);
+}
+
 void KateRenderer::paintCaret(const KTextEditor::Cursor &cursor, const KateLineLayoutPtr &range, QPainter &paint, int xStart, int xEnd)
 {
     if (range->includesCursor(cursor)) {
@@ -987,7 +1086,7 @@ void KateRenderer::paintCaret(const KTextEditor::Cursor &cursor, const KateLineL
                     width = inlineNote.width() + (caretStyle() == Line ? 2.0 : 0.0);
                 }
             }
-            range->layout()->drawCursor(&paint, QPoint(-xStart - width, 0), cursor.column(), caretWidth);
+            drawCursor(*range->layout(), &paint, QPoint(-xStart - width, 0), cursor.column(), caretWidth, lineHeight());
         } else {
             // Off the end of the line... must be block mode. Draw the caret ourselves.
             const KateTextLayout &lastLine = range->viewLine(range->viewLineCount() - 1);
@@ -1172,6 +1271,30 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
     // Syntax highlighting, inbuilt and arbitrary
     QVector<QTextLayout::FormatRange> decorations = decorationsForLine(textLine, lineLayout->line());
 
+    // Qt works badly if you have RTL text and formats set on that text.
+    // It will shape the text according to the given format ranges which
+    // produces incorrect results as a letter in RTL can have a different
+    // shape depending upon where in the word it resides. The resulting output
+    // looks like: وقار vs وق ار, i.e, the ligature قا is broken into ق ا which
+    // is really bad for readability
+    if (opt.textDirection() == Qt::RightToLeft) {
+        // We can fix this to a large extent here by using QGlyphRun etc, but for now
+        // we only fix this for formats which have a background color and a foreground
+        // color that is same as "dsNormal". Reasoning is that, it is unlikely that RTL
+        // text will have a lot of cases where you have partially colored ligatures. BG
+        // formats are different, you can easily have a format that covers a ligature partially
+        // as a result of "Search" or "multiple cursor selection"
+        QColor c = view()->theme().textColor(KSyntaxHighlighting::Theme::Normal);
+        decorations.erase(std::remove_if(decorations.begin(),
+                                         decorations.end(),
+                                         [c](const QTextLayout::FormatRange &fr) {
+                                             return fr.format.hasProperty(QTextFormat::BackgroundBrush)
+                                                 && (fr.format.property(QTextFormat::ForegroundBrush).value<QBrush>().color() == c
+                                                     || fr.format.foreground() == Qt::NoBrush);
+                                         }),
+                          decorations.end());
+    }
+
     int firstLineOffset = 0;
 
     if (!isPrinterFriendly() && m_view) {
